package com.yihu.wlyy.util;

import org.apache.log4j.Logger;
import org.apache.commons.net.ftp.*;
import java.io.*;

/**
 * Created by zdm on 2019/5/7.
 */
public class FtpFilesUtil {
    static Logger Log = Logger.getLogger(FtpFilesUtil.class);
    private static   FTPClient ftpClient;
    private static String ENCODING = "GBK";
    FTPClientConfig ftpConfig = new FTPClientConfig("UNIX");

    /**
     * 连接到ftp
     * @param address ip
     * @param port 端口
     * @param name 用户名
     * @param passwd 密码
     * @return
     */
    public boolean connectServer(String address,int port,String name,String passwd) {
        boolean flag = false;
        try {
            ftpClient = new FTPClient();
            ftpConfig.setServerLanguageCode("ISO-8859-1");
            ftpClient.connect(address, port);
            if(ftpClient.login(name,passwd)){
                if (FTPReply.isPositiveCompletion(ftpClient.sendCommand("OPTS UTF8", "ON"))) {
                    ENCODING = "UTF-8";
                }
                ftpClient.setControlEncoding(ENCODING);
                ftpClient.enterLocalPassiveMode();
                ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
                ftpClient.setFileTransferMode(FTP.STREAM_TRANSFER_MODE);
                ftpClient.setDataTimeout(120000);
            }
            int reply = ftpClient.getReplyCode();
            if (FTPReply.isPositiveCompletion(reply)) {
                flag = true;
                Log.info("FTP connect success!");
            } else {
                Log.warn("FTP refused to connect!");
                ftpClient.disconnect();
            }
        } catch (Exception e) {
            Log.error("Failed to login ftp " + address + ":" + port, e);
        }
        return flag;
    }

    /**
     * 判断服务器上是否存在该文件
     * @param path
     * @return
     * @throws IOException
     */
    public boolean existFile(String path) throws IOException {
        boolean flag = false;
        FTPFile[] ftpFileArr = ftpClient.listFiles(path);
        if (ftpFileArr.length > 0) {
            flag = true;
        }else {
            Log.info("file ‘"+path+ "' does not exist ! ");
        }
        return flag;
    }

    /**
     * 上传文件到服务器
     * @param remoteFilePath 远程文件路径： /data/rec2/2019/0507/08/52.mp3
     * @param uploadFile 待上传文件： D:\testFile\52.mp3
     * @return
     * @throws IOException
     */
    public boolean uploadFile(String remoteFilePath, File uploadFile) throws IOException {
        boolean flag = false;
        InputStream input = null;
        try {
            input = new FileInputStream(uploadFile);
            String remote = new String(remoteFilePath.getBytes(ENCODING), "ISO-8859-1");
            if (ftpClient.appendFile(remote, input)) {
                flag = true;
            }
        } finally {
            input.close();
        }
        Log.info("push file (" + uploadFile.getCanonicalPath() + ") => " + (flag ? "SUCCESS" : "FAILED"));
        return flag;
    }

    /**
     * 关闭服务器连接
     */
    public void closeConnect() {
        try {
            if (ftpClient != null) {
                ftpClient.logout();
                ftpClient.disconnect();
                Log.info("FTP connect closed!");
            }
        } catch (Exception e) {
            Log.info("Failed to close FTP connection!");
            e.printStackTrace();
        }
    }

    /**
     * 下载文件
     * @param remoteFilePath 远程文件：/data/rec2/2019/0507/08/52.mp3
     * @param downloadFile 下载到本地的文件：D:\testFile\52.mp3
     * @return
     * @throws IOException
     */
    public boolean downloadFile(String remoteFilePath, File downloadFile) throws IOException {
        boolean flag = false;
        OutputStream out = null;
        InputStream in=null;
        try {
            //获取文件流
            in = ftpClient.retrieveFileStream(remoteFilePath);
            // 文件读取方式一
            out = new FileOutputStream(downloadFile);
            int i;
            byte[] bytes = new byte[1024];
            while ((i = in.read(bytes)) != -1) {
                out.write(bytes, 0, i);
            }
            out.flush();
        }catch (IOException e){
            Log.info("File download failed :"+e.getMessage());
            e.printStackTrace();
        }finally {
            try {
                if(null!=in){
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            try {
                if(null!=in){
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        Log.info("File Download Successful");
        return flag;
    }
}
